#!/usr/bin/python
# -*- coding: utf-8 -*-

# attention: pour les accents, ce fichier doit être impérativement
#            lu et modifié sous éditeur avec encodage UTF-8

################################################################
######                                                    ######
######          Module additionnel: temps.py              ######
######                                                    ######
######            accompagnant la CALCULEXT               ######
######      [Calculatrice Extensible Version 1.40]        ######
######                                                    ######
################################################################

# Copyright (C) 2008 Jean-Paul Vidal dit "Tyrtamos" — Tous droits réservés.
#   Logiciel distribué sur le site <http://linux.jpvweb.com>
#
#   Ce programme est un logiciel libre. Vous pouvez le redistribuer ou le
#   modifier suivant les termes de la "GNU General Public License" version 3,
#   telle que publiée par la Free Software Foundation.
#
#   Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
#   AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
#   ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
#   Publique GNU pour plus de détails.
#
#   Ce programme doit être distribué avec une copie de la
#   License Générale Publique GNU (fichier license.html).
#   Si ce n’est pas le cas, consultez: <http://www.gnu.org/licenses/gpl.html>.

##################################################################
# Attention: configuration volontairement inhabituelle pour python
# les divisions entre 2 entiers seront décimales et pas entières!
# par exemple: 10/3 donnera "3.33333333333333" et pas "3"
# (les divisions entières seront désormais faites avec "//")
#
from __future__ import division

##################################################################
# Importation des modules système
#
import locale
import time
import datetime
from socket import *
import struct

from sys import modules
pmain = modules['__main__']

##################################################################
# Importation des modules nécessaires aux fonctions du présent module
#   (à compléter si nécessaire)
#
from math import *

###############################################################################
# Commande dont l'exécution est nécessaire lors du chargement du module
# pour avoir la présentation des dates en francais
#
locale.setlocale(locale.LC_TIME,'')

###############################################################################
# Fonctions dates et heures
###############################################################################

###############################################################################
# Donne la date et l'heure du jour ainsi que le jour de semaine
# sous le format ex: 'vendredi 19/10/2007 05:56:35'
#
def tempsordi():
    """tempsordi(): Donne la date, l'heure et le jour de semaine de l'ordinateur"""
    return  time.strftime('%A %d/%m/%Y %H:%M:%S')

###############################################################################
# Donne la date et l'heure exacte (à la seconde) par consultation d'un serveur ntp
#   nécessite donc un accès internet
# serveur ntp par defaut: ntp.univ-lyon1.fr
# autres serveurs France: voir http://www.cru.fr/services/ntp/serveurs_francais
# exemple de réponse de la fonction: 'samedi 10/11/2007 9:12:51'
#   (vous pourriez avoir aussi le numéro du jour dans l'année avec "str(t[7]))"
# vérif: l'heure est bien exacte à la sec près par rapport à une horloge "radio"
#
def tempsntp(sntp='ntp.univ-lyon1.fr'):
    """tempsntp(sntp='ntp.univ-lyon1.fr'): Donne la date et l'heure exacte par consultation d'un serveur ntp"""
    jsem=["lundi", "mardi", "mercredi", "jeudi", "vendredi", "samedi", "dimanche"]
    # nb de sec entre 1/1/1900 (=temps ntp) et 1/1/1970 (=temps python (=epoch))
    #    (calcul ici par: diffdate("1/1/1900","1/1/1970")*24*3600)
    # Vérification des données
    if not pmain.estchaine(sntp):
        raise ValueError ("erreur tempsntp(sntp='ntp.univ-lyon1.fr'): sntp doit être une chaine")
    # Calcul
    temps19701900 = 2208988800L
    buffer=1024
    # initialisation d'une connexion UDP
    client=socket(AF_INET, SOCK_DGRAM)
    # envoie de la requête UDP
    data='\x1b' + 47 * '\0'
    client.sendto(data, (sntp, 123))
    # réception de la réponse UDP
    data, addresse = client.recvfrom(buffer)
    if data:
        tps = struct.unpack('!12I', data)[10]
        tps -= temps19701900
        t=time.localtime(tps)
        ch=jsem[t[6]]+' '+str(t[2])+'/'+str(t[1])+'/'+str(t[0])+' '+str(t[3])+':'+str(t[4])+':'+str(t[5])
        return ch
    else:
        return "échec: pas de réponse du serveur"

###############################################################################
# Fonctions heures
###############################################################################

###############################################################################
# Donne l'heure courante sous le format ex: '05:57:56'
#
def cetteheure():
    """cetteheure(): donne l'heure courante de l'ordinateur sous le format ex: '05:57:56'"""
    return time.strftime('%X')

###############################################################################
# Donne l'heure en "hh:mm:ss.sss" a partir des secondes decimales
#
def sec2hms(sd):
    """sec2hms(sd): donne l'heure en "hh:mm:ss.sss" à partir des secondes décimales"""
    # Vérification de la donnée
    if (not pmain.estnombre(sd)) or (sd<0):
        raise ValueError ("erreur sec2hms(sd): sd doit être un nombre positif ou nul")
    # Calcul
    h=0
    m=0
    s=sd
    if s>60:
        m=int(s/60)
        s=s-m*60
        if m>60:
            h=int(m/60)
            m=m-h*60
    hch=str(h)
    hch=hch.zfill(2)
    mch=str(m)
    mch=mch.zfill(2)
    sch=str(round(s,3))
    if s<10:
        sch="0"+sch
    return hch+":"+mch+":"+sch

###############################################################################
# Fonctions dates
###############################################################################

###############################################################################
# Donne la date du jour sous le format 'jj/mm/aaaa' (ex: '20/10/2007')
#
def cejour():
    """cejour(): donne la date du jour sous le format 'jj/mm/aaaa' (ex: '20/10/2007')"""
    return time.strftime('%d/%m/%Y')

###############################################################################
# Dit si l'année donnée "a" (aaaa) est bissextile ou non
#
def _bissextile(an):
    """_bissextile(an): dit si l'année donnée est bissextile ou non (True=oui, False=non)"""
    if (an % 4)==0:
        if ((an % 100)==0) and ((an % 400)<>0):
            return False
        else:
            return True
    else:
        return False

def bissextile(an):
    """bissextile(an): dit si l'année donnée est bissextile ou non (True=oui, False=non)"""
    # Vérification de la donnée
    if (not pmain.estnombre(an)) or (not pmain.estentier(an)) or (an<=0):
        raise ValueError ("erreur bissextile(an): an doit être un nombre entier positif")
    return _bissextile(an)

###############################################################################
# Transforme une date liste [j,m,a] en une date chaîne 'jj/mm/aaaa'
# suppose que la date j,m,a est valide et ne la vérifie pas
#
def datechaine(L):
    """datechaine(L): transforme une date liste=[j,m,a] en une date chaîne 'jj/mm/aaaa'"""
    return repr(L[0]).zfill(2) + "/" + repr(L[1]).zfill(2) + "/" + repr(L[2]).zfill(4)

###############################################################################
# Transforme une date chaîne 'j/m/a' en une date liste [j,m,a]
# suppose que la date j,m,a est valide et ne la vérifie pas
#
def dateliste(D):
    """dateliste(D): transforme une date chaîne 'jj/mm/aaaa' en une date liste [j,m,a]"""
    L=D.split('/')
    return [int(L[0]),int(L[1]),int(L[2])]

###############################################################################
# Vérifie si une date j,m,a est valide, et renvoie True ou False
#
def verifdate(j,m,a):
    """verifdate(j,m,a): vérifie si une date j,m,a est valide et génère une erreur si ce n'est pas le cas"""
    if a<1:
        raise ValueError ("erreur verifdate(j,m,a): l'année de la date est invalide (<1)")
    if m<1:
        raise ValueError ("erreur verifdate(j,m,a): le mois de la date est invalide (<1)")
    if m>12:
        raise ValueError ("erreur verifdate(j,m,a): le mois de la date est invalide (>12)")
    if j<1:
        raise ValueError ("erreur verifdate(j,m,a): le jour de la date est invalide (<1)")
    if bissextile(a):
        if j>(31,29,31,30,31,30,31,31,30,31,30,31)[m-1]:
            raise ValueError ("erreur verifdate(j,m,a): le jour de la date est invalide (trop grande)")
    else:
        if j>(31,28,31,30,31,30,31,31,30,31,30,31)[m-1]:
            raise ValueError ("erreur verifdate(j,m,a): le jour de la date est invalide (trop grande)")

###############################################################################
# Transforme la date chaîne D 'j/m/a' en une date chaîne 'jj/mm/aaaa' et la vérifie
#
def normaldate(D):
    """normaldate(D): transforme la date chaîne D 'j/m/a' en une date chaîne 'jj/mm/aaaa'  et la vérifie"""
    # verif structure chaîne
    if not pmain.estchaine(D):
        raise ValueError ("erreur normaldate(D): D doit être une chaîne")
    for i in range(0,len(D)):
        if (not D[i].isdigit()) and (D[i]<>"/"):
            raise ValueError ("erreur normaldate(D): caractères anormaux dans D")
    L=D.split('/')
    if len(L)<>3:
        raise ValueError ("erreur normaldate(D): il faut 2 séparateurs '/'")
    if L[0]=="":
        raise ValueError ("erreur normaldate(D): manque le jour")
    if L[1]=="":
        raise ValueError ("erreur normaldate(D): manque le mois")
    if L[2]=="":
        raise ValueError ("erreur normaldate(D): manque l'année")
    # vérif validité de la date
    verifdate(int(L[0]),int(L[1]),int(L[2]))
    # retourne la date vérifiée et sous le bon format 'jj/mm/aaaa'
    return L[0].zfill(2) + '/' + L[1].zfill(2) + '/' + L[2].zfill(4)

###############################################################################
# Donne le nombre de jour de l'annee a
#
def _nbjoursan(an):
    """_nbjoursan(an): donne le nombre de jours de l'année"""
    if bissextile(an):
        return 366
    else:
        return 365

def nbjoursan(an):
    """nbjoursan(an): donne le nombre de jours de l'année"""
    # Vérification de la donnée
    if (not pmain.estnombre(an)) or (not pmain.estentier(an)) or (an<=0):
        raise ValueError ("erreur nbjoursan(an): a doit être un nombre entier positif")
    return _nbjoursan(an)

###############################################################################
# Donne le numéro du jour dans l'année d'une date D 'jj/mm/aaaa' (1er janvier = 1, ...)
#
def _numjouran(D):
    """_numjouran(D): donne le numéro du jour dans l'année  de la date D 'jj/mm/aaaa' (1er janvier = 1, ...)"""
    L=D.split('/')
    if bissextile(int(L[2])):
        return (0,31,60,91,121,152,182,213,244,274,305,335,366)[int(L[1])-1] + int(L[0])
    else:
        return (0,31,59,90,120,151,181,212,243,273,304,334,365)[int(L[1])-1] + int(L[0])

def numjouran(D):
    """numjouran(D): donne le numéro du jour dans l'année  de la date D 'jj/mm/aaaa' (1er janvier = 1, ...)"""
    D=normaldate(D)
    return _numjouran(D)

###############################################################################
# Donne la date qui est le nième jour de l'année an
#
def _datenumjouran(n,an):
    """_datenumjouran(n,an): donne la date qui est le nième jour de l'année an"""
    if bissextile(an):
        jm=(0,31,60,91,121,152,182,213,244,274,305,335,366)
    else:
        jm=(0,31,59,90,120,151,181,212,243,273,304,334,365)
    for i in range(1,13):
        if jm[i]>=n:
            m=i
            break
    j=n-jm[m-1]
    return repr(j).zfill(2) + "/" + repr(m).zfill(2) + "/" + repr(an).zfill(4)

def datenumjouran(n,an):
    """datenumjouran(n,an): donne la date qui est le nième jour de l'année an"""
    # Vérification des données
    if (not pmain.estnombre(n)) or (not pmain.estentier(n)) or (n<=0):
        raise ValueError ("erreur datenumjouran(n,an): n doit être un nombre entier positif")
    if (not pmain.estnombre(an)) or (not pmain.estentier(an)) or (an<=0):
        raise ValueError ("erreur datenumjouran(n,an): an doit être un nombre entier positif")
    return _datenumjouran(n,an)

###############################################################################
# Dit si une date D2 'j/m/a' est postérieure ou égale à une autre date D1 'j/m/a'
#
def _postdate(D1,D2):
    """_postdate(D1,D2): dit si une date D2 'jj/mm/aaaa' est postérieure ou égale à une autre date D1 'jj/mm/aaaa'"""
    L1=D1.split('/')
    L2=D2.split('/')
    if int(L2[2])>int(L1[2]):
        return True
    elif int(L2[2])==int(L1[2]):
        if int(L2[1])>int(L1[1]):
            return True
        elif int(L2[1])==int(L1[1]):
            if int(L2[0])>=int(L1[0]):
                return True
    return False

def postdate(D1,D2):
    """postdate(D1,D2): dit si une date D2 'j/m/a' est postérieure ou égale à une autre date D1 'j/m/a'"""
    D1=normaldate(D1)
    D2=normaldate(D2)
    return _postdate(D1,D2)

###############################################################################
# Donne le nombre de jours entre 2 dates 'j/m/a'
# Si D2 postérieure ou égale a D1, renvoie un nb positif ou nul
# sinon, renvoie un nb négatif
#
def _diffdate(D1,D2):
    """_diffdate(D1,D2): nombre de jours entre 2 dates ('jj/mm/aaaa'), >=0 si D2>=D1, <0 sinon"""
    L1=D1.split('/')
    L2=D2.split('/')
    a1=int(L1[2])
    a2=int(L2[2])
    x1=numjouran(D1)
    x2=numjouran(D2)
    if postdate(D1,D2):
        # D2 est postérieure a D1: retour d'un nb >=0
        nk=x2-x1
        for ak in range(a1,a2):
            nk=nk+nbjoursan(ak)
        return nk
    else:
        # D2 est postérieure a D1: retour d'un nb >=0
        nk=x2-x1
        for ak in range(a2,a1):
            nk=nk-nbjoursan(ak)
        return nk

def diffdate(D1,D2):
    """diffdate(D1,D2): nombre de jours entre 2 dates ('j/m/a'), >=0 si D2>=D1, <0 sinon"""
    # Vérification des données
    D1=normaldate(D1)
    D2=normaldate(D2)
    # Calcul
    return _diffdate(D1,D2)

###############################################################################
# Donne la date postérieure de n jours(n>=0) ou antérieure de n jours (n<0) à la date D 'j/m/a'
#
def _adddate(D,n):
    """_adddate(D,n): donne la date postérieure de n jours (n>=0) ou antérieure de n jours (n<0) à la date D 'jj/mm/aaaa'"""
    L=D.split('/')
    ak=int(L[2])
    if n>=0:
        nk=numjouran(D)+n
        while nk>nbjoursan(ak):
            nk=nk-nbjoursan(ak)
            ak=ak+1
        return datenumjouran(nk,ak)
    else:
        nk=abs(n)+nbjoursan(ak)-numjouran(D)
        while nk>=nbjoursan(ak):
            nk=nk-nbjoursan(ak)
            ak=ak-1
        return datenumjouran(nbjoursan(ak)-nk,ak)

def adddate(D,n):
    """adddate(D,n): donne la date postérieure de n jours (n>=0) ou antérieure de n jours (n<0) à la date D 'j/m/a'"""
    # Vérification des données
    if (not pmain.estnombre(n)) or (not pmain.estentier(n)):
        raise ValueError ("erreur adddate(D,n): n doit être un nombre entier")
    D=normaldate(D)
    # Calcul
    return _adddate(D,n)

###############################################################################
# Donne le numéro du jour de la semaine d'une date D 'j/m/a' (lundi=1, ...)
#
def _numjoursem(D):
    """_numjoursem(D): donne le numéro du jour de la semaine d'une date D 'jj/mm/aaaa' (lundi=1, ...)"""
    L=D.split('/')
    an=int(L[2])-1
    j=(an+(an//4)-(an//100)+(an//400)+numjouran(D)) % 7
    if j==0: j=7
    return j

def numjoursem(D):
    """numjoursem(D): donne le numéro du jour de la semaine d'une date D 'j/m/a' (lundi=1, ...)"""
    # Vérification des données
    D=normaldate(D)
    # Calcul
    return _numjoursem(D)

###############################################################################
# Donne le jour de la semaine d'une date D 'j/m/a' ('lundi' pour lundi, ...)
#
def _joursem(D):
    """_joursem(D): donne le jour de la semaine d'une date D 'j/m/a' ('lundi' pour lundi, ...)"""
    return ('lundi', 'mardi', 'mercredi', 'jeudi', 'vendredi', 'samedi', 'dimanche')[_numjoursem(D)-1]

def joursem(D):
    """joursem(D): donne le jour de la semaine d'une date D 'j/m/a' ('lundi' pour lundi, ...)"""
    # Vérification des données
    D=normaldate(D)
    # Calcul
    return _joursem(D)

###############################################################################
# Trouve la date du lundi d'une semaine qui contient une date D 'j/m/a'
#
def _lundi(D):
    """_lundi(D): trouve la date du lundi d'une semaine qui contient la date D 'j/m/a'"""
    return adddate(D,-numjoursem(D)+1)

def lundi(D):
    """lundi(D): trouve la date du lundi d'une semaine qui contient la date D 'j/m/a'"""
    # Vérification des données
    D=normaldate(D)
    # Calcul
    return _lundi(D)

###############################################################################
# calcul de la date du début du mois suivant
# exemple: debmoissuivant('15/7/2008') => '1/8/2008'
#
def debmoissuivant(D):
    """debmoissuivant(D): donne la date '01/mm/aaaa' au 1er du mois qui suit D 'j/m/a'"""
    (J,M,A) = D.split('/')
    MI=int(M)
    MI+=1
    if MI==13:
        MI=1
    return "%02d/%02d/%04d" % (1, MI, int(A))

###############################################################################
# dit si une date D donnée 'j/m/a' correspond à un samedi ou un dimanche
#
def _estweekend(D):
    """_estweekend(D): dit si une date D donnée 'jj/mm/aaaa' correspond à un samedi ou un dimanche"""
    if numjoursem(D)>5:
        return True
    else:
        return False

def estweekend(D):
    """estweekend(D): dit si une date D donnée 'j/m/a' correspond à un samedi ou un dimanche"""
    # Vérification des données
    D=normaldate(D)
    # Calcul
    return _estweekend(D)

###############################################################################
# Trouve la date de Pâques d'une année donnée an
#  méthode Hodges: validité = calendrier grégorien de 1583 a 4100
#     (verif faite avec toutes les dates officielles)
#  voir méthode ici: http://users.chariot.net.au/~gmarts/eastalg.htm#EasterHodges
#  et pour vérif ici: http://www.dioceserimouski.com/ch/paquesdates.html
#
def _paques(an):
    """_paques(an): trouve la date de Pâques d'une année donnée an"""
    a=an//100
    b=an%100
    c=(3*(a+25))//4
    d=(3*(a+25))%4
    e=(8*(a+11))//25
    f=(5*a+b)%19
    g=(19*f+c-e)%30
    h=(f+11*g)//319
    j=(60*(5-d)+b)//4
    k=(60*(5-d)+b)%4
    m=(2*j-k-g+h)%7
    n=(g-h+m+114)//31
    p=(g-h+m+114)%31
    jour=p+1
    mois=n
    return repr(jour).zfill(2) + "/" + repr(mois).zfill(2) + "/" + repr(an).zfill(4)

def paques(an):
    """paques(an): trouve la date de Pâques d'une année donnée an"""
    # Vérification des données
    if (not pmain.estnombre(an)) or (not pmain.estentier(an)) or (an<1583) or (an>4100):
        raise ValueError ("erreur paques(an): an doit être un nombre entier >=1583 et <=4100")
    return _paques(an)

###############################################################################
# Trouve la date du jeudi de l'ascension (paques + 39j)
#  (voir ici: http://fr.wikipedia.org/wiki/Calendrier_chr%C3%A9tien)
#
def ascension(an):
    """ascension(an): trouve la date du jeudi de l'ascension (paques + 39j)"""
    return adddate(paques(an),39)

###############################################################################
# Trouve la date du dimanche de la Pentecôte (Pâques + 49j)
#  (voir ici: http://fr.wikipedia.org/wiki/Calendrier_chr%C3%A9tien)
#
def pentecote(an):
    """pentecote(an): trouve la date du dimanche de la Pentecôte (paques + 49j)"""
    return adddate(paques(an),49)

###############################################################################
# Renvoie la liste des jours fériés (France) de l'année donnée en 1er paramètre
#   Le 2ème paramètre numérique est optionnel:
#   - avec "0", les samedis et dimanches sont exclus
#   - avec paramètre non donné ou "1", on a en plus les samedis et dimanches
#   - avec "2", on a comme "0", mais avec, en plus, les 2 jours fériés d'Alsace-Moselle
#   - avec "3", on a comme "2", avec les samedis et dimanches
#   => à adapter selon votre situation, region, pays, besoins et évolution de la législation
#
def _feries(an,sd=1):
    """_feries(an,sd=1): => liste des jours fériés France de l'année an. Avec sd=0: sans les sammedis-dimanches. Avec sd=2: idem sd=0 + les 2 jours fériés supplémentaires d'Alsace-Moselle. Avec sd=3: idem sd=2 + samedis-dimanches"""
    F=[]
    DP=paques(an)
    A=str(an)

    X="01/01/" + A
    if (numjoursem(X)<6) or (sd==1) or (sd==3):
        F.append("Jour de l'an = " + joursem(X) + " " + X)

    if sd>=2:
        # Spécifique Alsace-Moselle: le vendredi saint est férié
        X=adddate(DP,-2)
        F.append("Vendredi Saint = " + joursem(X) + " " + X)

    if (sd==1) or (sd==3):
        F.append("Jour de Pâques = " + joursem(DP) + " " + DP)

    X=adddate(DP,1)
    F.append("lundi de Pâques= " + joursem(X) + " " + X)

    X="01/05/" + A
    if (numjoursem(X)<6) or (sd==1) or (sd==3):
        F.append("Fête du Travail = " + joursem(X) + " " + X)

    X="08/05/" + A
    if (numjoursem(X)<6) or (sd==1) or (sd==3):
        F.append("Fête de la Victoire 1945 = " + joursem(X) + " " + X)

    X=adddate(DP,39)
    F.append("Ascension = "  + joursem(X) + " " + X)

    X=adddate(DP,49)
    if  (sd==1) or (sd==3):
        F.append("Pentecôte = " + joursem(X) + " " + X)

    X=adddate(DP,50)
    F.append("Lundi de Pentecôte = " + joursem(X) + " " + X)

    X="14/07/" + A
    if (numjoursem(X)<6) or (sd==1) or (sd==3):
        F.append("Fête Nationale = " + joursem(X) + " " + X)

    X="15/08/" + A
    if (numjoursem(X)<6) or (sd==1) or (sd==3):
        F.append("Assomption = " + joursem(X) + " " + X)

    X="01/11/" + A
    if (numjoursem(X)<6) or (sd==1) or (sd==3):
        F.append("Toussaint = " + joursem(X) + " " + X)

    X="11/11/" + A
    if (numjoursem(X)<6) or (sd==1) or (sd==3):
        F.append("Armistice 1918 = " + joursem(X) + " " + X)

    X="25/12/" + A
    if (numjoursem(X)<6) or (sd==1) or (sd==3):
        F.append("Noël = " + joursem(X) + " " + X)

    if sd>=2:
        # Spécifique Alsace-Moselle: la St-Etienne, lendemain de Noël, est fériée
        X="26/12/" + A
        if (numjoursem(X)<6) or (sd==3):
            F.append("St Etienne = " + joursem(X) + " " + X)

    return F

def feries(an,sd=1):
    """feries(an,sd=1): => liste des jours fériés France de l'année an. Avec sd=0: sans les sammedis-dimanches. Avec sd=2: idem sd=0 + les 2 jours fériés supplémentaires d'Alsace-Moselle. Avec sd=3: idem sd=2 + samedis-dimanches"""
    # Vérification des données
    if (not pmain.estnombre(an)) or (not pmain.estentier(an)):
        raise ValueError ("erreur feries(an,sd=1): an doit être un nombre entier")
    if (not pmain.estnombre(sd)) or (not pmain.estentier(sd)) or (sd<0) or (sd>3):
        raise ValueError ("erreur feries(an,sd=1): sd doit être un nombre entier = 0, 1, 2 ou 3")
    return _feries(an,sd)


###############################################################################
# Dit si une date D 'j/m/a' est fériée France (True=vrai, False=faux)
# Avec sd>1, on a les 2 jours fériés supplémentaires d'Alsace-Moselle
# cette fonction ne neutralise pas les jours fériés qui tombent un samedi ou un dimanche
#
def _estferie(D,sd=1):
    """estferie(D,sd=1): => dit si une date D donnée 'jj/mm/aaaa' est fériée France. Avec sd>1, on a en plus les 2 jours fériés supplémentaires d'Alsace-Moselle"""
    DJM=D[0:5]
    DP=paques(int(D[6:]))
    # test jour de l'an
    if DJM=="01/01": return True
    if sd>1:
        # test Vendredi Saint pour Alsace-Moselle
        if D==adddate(DP,-2): return True
    # test Dimanche de Pâques
    if D==DP: return True
    # test lundi de Pâques
    if D==adddate(DP,1): return True
    # test 1er mai
    if DJM=="01/05": return True
    # test 8 mai
    if DJM=="08/05": return True
    # test jeudi de l'ascention
    if D==adddate(DP,39): return True
    # test dimanche de la Pentecôte
    if D==adddate(DP,49): return True
    # test lundi de la Pentecôte
    if D==adddate(DP,50): return True
    # test 14 juillet
    if DJM=="14/07": return True
    # test 15 aout
    if DJM=="15/08": return True
    # test 1er novembre
    if DJM=="01/11": return True
    # test 11 novembre
    if DJM=="11/11": return True
    # test jour de Noël
    if DJM=="25/12": return True
    if sd>1:
        # test St-Etienne pour Alsace-Moselle
        if DJM=="26/12": return True
    return False

def estferie(D,sd=1):
    """estferie(D,sd=1): => dit si une date D donnée 'j/m/a' est fériée France. Avec sd>1, on a en plus les 2 jours fériés supplémentaires d'Alsace-Moselle"""
    # Vérification des données
    if (not pmain.estnombre(sd)) or (not pmain.estentier(sd)) or (sd<0):
        raise ValueError ("erreur estferie(D,sd=1): sd doit être un nombre entier >= 0")
    D=normaldate(D)
    # Calcul
    return _estferie(D,sd)


###############################################################################
# calcule le nombre de jours ouvrés France entre 2 dates 'j/m/a'. Si sd>1: calcul pour l'Alsace-Moselle
# = neutralise les samedis, dimanches et jours fériés France
#
def _diffdateouvre(D1,D2,sd=1):
    """_diffdateouvre(D1,D2,sd=1): calcule le nombre de jours ouvrés entre 2 dates. Si sd>1: calcul pour l'Alsace-Moselle"""
    n=_diffdate(D1,D2)
    if n<0:
        D1,D2=D2,D1
    x=0
    for i in xrange(0,abs(n)):
        D=_adddate(D1,i)
        if not (_estweekend(D) or _estferie(D,sd)):
            x+=1
    if n<0:
        return -x
    else:
        return x

def diffdateouvre(D1,D2,sd=1):
    """diffdateouvre(D1,D2,sd=1): calcule le nombre de jours ouvrés entre 2 dates. Si sd>1: calcul pour l'Alsace-Moselle"""
    # Vérification des données
    if (not pmain.estnombre(sd)) or (not pmain.estentier(sd)) or (sd<0):
        raise ValueError ("erreur diffdateouvre(D1,D2,sd=1): sd doit être un nombre entier >= 0")
    D1=normaldate(D1)
    D2=normaldate(D2)
    return _diffdateouvre(D1,D2,sd)

###############################################################################
# Calcul d'une date 'jj/mm/aaaa' à partir du jour julien des éphémérides JJ
#  réf: livre “Calculs astronomiques à l'usage des amateurs” de Jean Meeus,
#           édité par la Société Astronomique de France
#
def jj2date(JJ):
    """jj2date(JJ): calcul d'une date 'jj/mm/aaaa' à partir du jour julien des éphémérides JJ"""
    JJ += 0.5
    Z = int(JJ)
    F = JJ - Z
    if Z < 2299161:
        A = Z
    else:
        alpha = int((Z - 1867216.25) / 36524.25)
        A = Z + 1 + alpha - int(alpha / 4)
    B = A + 1524
    C = int((B - 122.1) / 365.25)
    D = int(365.25 * C)
    E = int((B - D) /30.6001)

    JD = B - D - int(30.6001 * E) + F          # calcul du jour décimal

    J = int(JD)                                # calcul du jour

    """
    # en reserve: non utilisé pour l'instant
    HD = 24*(JD-J)
    H = int(HD)                                # calcul des heures
    MND = 60*(HD-H)
    MN = int(MND)                              # calcul des minutes
    S = int(60*(MND-MN))                       # calcul des secondes entières
    """

    if (E < 13.5):                             # calcul du mois
        M = E - 1
    else:
        M = E - 13

    if (M > 2.5):                              # calcul de l'année
        A = C - 4716
    else:
        A = C - 4715

    return "%02d/%02d/%04d" % (J, M, A)

###############################################################################
# Calcul de la date de la phase de la lune correspondant à la valeur k
#  réf: livre “Calculs astronomiques à l'usage des amateurs” de Jean Meeus,
#           édité par la Société Astronomique de France
#
def calculphaseslune(k):
    """calculphaseslune(): calcul de la date de la phase de la lune correspondant à la valeur k"""

    # calcul de T en fonction de k (formule 26.3 du livre de Jean MEEUS)
    T = k / 1236.85

    # calcul de JJ = phase moyenne de la lune (formule 26.1 du livre de Jean MEEUS)
    JJ = 2415020.75933 + 29.53058868*k  + 0.0001178*T*T - 0.000000155*T*T*T   \
        + 0.00033 * sin(radians(166.56) + radians(132.87)*T - radians(0.009173)*T*T)

    # anomalie moyenne du soleil à l'instant JJ
    M = 359.2242 + 29.10535608*k - 0.0000333*T*T - 0.00000347*T*T*T

    # anomalie moyenne de la lune  à l'instant JJ
    MP = 306.0253 + 385.81691806*k + 0.0107306*T*T + 0.00001236*T*T*T

    # argument de la latitude de la lune  à l'instant JJ
    F = 21.2964 + 390.67050646*k - 0.0016528*T*T - 0.00000239*T*T*T

    # application de corrections selon les décimales de k
    kp = int(round((k - int(k))*100,0))
    if (kp==0) or (kp==50):
        # => nouvelle lune ou pleine lune
        JJ += (0.1734 - 0.000393*T) * sin(radians(M))  \
                + 0.0021 * sin(radians(2*M)) \
                - 0.4068 * sin(radians(MP))  \
                + 0.0161 * sin(radians(2*MP))  \
                - 0.0004 * sin(radians(3*MP))  \
                + 0.0104 * sin(radians(2*F))  \
                - 0.0051 * sin(radians(M+MP))  \
                - 0.0074 * sin(radians(M-MP))  \
                + 0.0004 * sin(radians(2*F+M))  \
                - 0.0004 * sin(radians(2*F-M))  \
                - 0.0006 * sin(radians(2*F+MP))  \
                + 0.0010 * sin(radians(2*F-MP))  \
                + 0.0005 * sin(radians(M+2*MP))
    else:
        JJ += (0.1721 - 0.0004*T) * sin(radians(M))  \
                + 0.0021 * sin(radians(2*M))  \
                - 0.6280 * sin(radians(MP))  \
                + 0.0089 * sin(radians(2*MP))  \
                - 0.0004 * sin(radians(3*MP))  \
                + 0.0079 * sin(radians(2*F))  \
                - 0.0119 * sin(radians(M+MP))  \
                - 0.0047 * sin(radians(M-MP))  \
                + 0.0003 * sin(radians(2*F+M))  \
                - 0.0004 * sin(radians(2*F-M))  \
                - 0.0006 * sin(radians(2*F+MP))  \
                + 0.0021 * sin(radians(2*F-MP))  \
                + 0.0003 * sin(radians(M+2*MP))  \
                + 0.0004 * sin(radians(M-2*MP))  \
                - 0.0003 * sin(radians(2*M+MP))
        if kp==25:
            # => premier quartier
            JJ += 0.0028 - 0.0004*cos(radians(M)) + 0.0003*cos(radians(MP))
        else:
            # => dernier quartier
            JJ += 0.0028 + 0.0004*cos(radians(M)) - 0.0003*cos(radians(MP))

    # retour de la date du calendrier grégorien à partir du jour Julien des éphémérides
    return jj2date(JJ)

###############################################################################
# Utilitaire de calcul de k pour la date D 'j/m/a' (phases de la lune)
#  réf: livre “Calculs astronomiques à l'usage des amateurs” de Jean Meeus,
#           édité par la Société Astronomique de France
#
def _phaseslune(D):
    """_phaseslune(D): utilitaire de calcul de k pour la date D 'j/m/a' (phases de la lune)"""

    # extraction de la date
    lst=D.split('/')
    J = int(lst[0])
    M = int(lst[1])
    A = int(lst[2])

    # calcul de l'année décimale
    if bissextile(A):
        A += ((0,31,60,91,121,152,182,213,244,274,305,335,366)[M-1] + J) / 366
    else:
        A += ((0,31,59,90,120,151,181,212,243,273,304,334,365)[M-1] + J) / 365

    # renvoi de k
    return (A-1900)*12.3685

###############################################################################
# Calcul des n dates de phase de la lune qui commence(nt) à la date D 'j/m/a'
#  réf: livre “Calculs astronomiques à l'usage des amateurs” de Jean Meeus,
#           édité par la Société Astronomique de France
#
def phaseslune(D, n=1):
    """phaseslune(D, n=1): calcul des n dates de phase de la lune qui commence(nt) à la date D 'j/m/a' """

    # signification de k:
    #    k entier => nouvelle lune
    #    k=xxx.25 => premier quartier
    #    k=xxx.50 => pleine lune
    #    k=xxx.75 => dernier quartier

    # Vérification des données
    D=normaldate(D)
    if (not pmain.estnombre(n)) or (not pmain.estentier(n)) and (n<1):
        raise ValueError ("erreur phaseslune(D, n=1): n doit être un nombre entier >=1")

    # calcul du k initial basé sur D
    k=_phaseslune(D)

    # calcul de la 1ère date (éventuellement antérieure à D)
    k=int(k)
    p=0
    d = [calculphaseslune(k), p]

    # trouver la 1ère date de phase lunaire postérieure ou égale à D
    while not postdate(D, d[0]):
        k+=0.25
        p = (p+1) % 4
        d = [calculphaseslune(k), p]
    L=[d]

    # si n>1, calculer autant de dates de phase lunaire que demandé
    for i in xrange(1,n):
        k+=0.25
        p = (p+1) % 4
        d = [calculphaseslune(k), p]
        L.append(d)

    # retour de la liste des dates cherchées avec le type de phase pour chaque date
    t=["nouvelle lune","premier quartier","pleine lune","dernier quartier"]
    R=[]
    for c, p in L:
        R.append([c, t[p]])

    return R

###############################################################################
# Calcul des dates de phase de la lune à partir de D1 'j/m/a' et jusqu'à D2 'j/m/a' exclue
#  réf: livre “Calculs astronomiques à l'usage des amateurs” de Jean Meeus,
#           édité par la Société Astronomique de France
#
def phaseslune2(D1, D2):
    """phaseslune2(D1,D2): calcul des dates de phase de la lune à partir de D1 'j/m/a' et jusqu'à D2 'j/m/a' exclue"""

    # signification de k:
    #    k entier => nouvelle lune
    #    k=xxx.25 => premier quartier
    #    k=xxx.50 => pleine lune
    #    k=xxx.75 => dernier quartier

    # Vérification des données
    D1 = normaldate(D1)
    D2 = normaldate(D2)

    # calcul du k initial basé sur D
    k=_phaseslune(D1)

    # calcul de la 1ère date (éventuellement antérieure à D)
    k=int(k)
    p=0
    d = [calculphaseslune(k), p]

    # trouver la 1ère date de phase lunaire postérieure ou égale à D
    while not postdate(D1, d[0]):
        k+=0.25
        p = (p+1) % 4
        d = [calculphaseslune(k), p]
    L=[d]

    if postdate(D2, d[0]):
        return [[]]

    # calculer autant de dates de phase lunaire qu'il y a jusqu'à D2 exclue
    while True:
        k+=0.25
        p = (p+1) % 4
        d = [calculphaseslune(k), p]
        if postdate(D2, d[0]):
            break
        else:
            L.append(d)

    # retour de la liste des dates cherchées avec le type de phase pour chaque date
    t=["nouvelle lune","premier quartier","pleine lune","dernier quartier"]
    R=[]
    for c, p in L:
        R.append([c, t[p]])

    return R

###############################################################################
#
